/*
	Copyright (C) 2006,2007 John Anderson

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#include "controls.h"

#include <iostream>
#include <iomanip>
#include <sstream>

using namespace Mackie;
using namespace std;

void Group::add( Control & control )
{
	controls.push_back( &control );
}

Surface::Surface( uint32_t max_strips )
: _max_strips( max_strips )
{
	strips.resize( max_strips );
	init_controls();
}

Surface::~Surface()
{
	// delete groups
	for( Groups::iterator it = groups.begin(); it != groups.end(); ++it )
	{
		delete it->second;
	}
	
	// delete controls
	for( Controls::iterator it = controls.begin(); it != controls.end(); ++it )
	{
		delete *it;
	}
}

ostream & Mackie::operator << ( ostream & os, const Mackie::Control & control )
{
	os << typeid( control ).name();
	os << " { ";
	os << "name: " << control.name();
	os << ", ";
	os << "id: " << "0x" << setw(2) << setfill('0') << hex << control.id() << setfill(' ');
	os << ", ";
	os << "ordinal: " << dec << control.ordinal();
	os << ", ";
	os << "group: " << control.group().name();
	
	return os;
}

/**
	TODO could optimise this to use enum, but it's only
	called during the protocol class instantiation.

	generated using

	irb -r controls.rb
	sf=Surface.new
	sf.parse
	controls = sf.groups.find{|x| x[0] =~ /strip/}.each{|x| puts x[1]}
	controls[1].each {|x| puts "\telse if ( control.name() == \"#{x.name}\" )\n\t{\n\t\t_#{x.name} = reinterpret_cast<#{x.class.name}*>(&control);\n\t}\n"}
*/
void Strip::add( Control & control )
{
	Group::add( control );
	if ( control.name() == "gain" )
	{
		_gain = reinterpret_cast<Fader*>(&control);
	}
	else if ( control.name() == "vpot" )
	{
		_vpot = reinterpret_cast<Pot*>(&control);
	}
	else if ( control.name() == "recenable" )
	{
		_recenable = reinterpret_cast<Button*>(&control);
	}
	else if ( control.name() == "solo" )
	{
		_solo = reinterpret_cast<Button*>(&control);
	}
	else if ( control.name() == "mute" )
	{
		_mute = reinterpret_cast<Button*>(&control);
	}
	else if ( control.name() == "select" )
	{
		_select = reinterpret_cast<Button*>(&control);
	}
	else if ( control.name() == "vselect" )
	{
		_vselect = reinterpret_cast<Button*>(&control);
	}
	else if ( control.name() == "fader_touch" )
	{
		_fader_touch = reinterpret_cast<Button*>(&control);
	}
	else
	{
		ostringstream os;
		os << "Unknown control type " << control;
		throw MackieControlException( os.str() );
	}
}
	
/**
 generated with

controls[1].each do |x|
  puts <<EOF
#{x.class.name} & Strip::#{x.name}()
{
\tif ( _#{x.name} == 0 )
\t\tthrow MackieControlException( "#{x.name} is null" );
\treturn *_#{x.name};
}
EOF
end
*/
Fader & Strip::gain()
{
	if ( _gain == 0 )
		throw MackieControlException( "gain is null" );
	return *_gain;
}
Pot & Strip::vpot()
{
	if ( _vpot == 0 )
		throw MackieControlException( "vpot is null" );
	return *_vpot;
}
Button & Strip::recenable()
{
	if ( _recenable == 0 )
		throw MackieControlException( "recenable is null" );
	return *_recenable;
}
Button & Strip::solo()
{
	if ( _solo == 0 )
		throw MackieControlException( "solo is null" );
	return *_solo;
}
Button & Strip::mute()
{
	if ( _mute == 0 )
		throw MackieControlException( "mute is null" );
	return *_mute;
}
Button & Strip::select()
{
	if ( _select == 0 )
		throw MackieControlException( "select is null" );
	return *_select;
}
Button & Strip::vselect()
{
	if ( _vselect == 0 )
		throw MackieControlException( "vselect is null" );
	return *_vselect;
}
Button & Strip::fader_touch()
{
	if ( _fader_touch == 0 )
		throw MackieControlException( "fader_touch is null" );
	return *_fader_touch;
}
