#! /usr/bin/ruby
# Copyright (C) 2006,2007 John Anderson

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

require 'faster_csv'

class String
  def to_b
    to_i != 0 || %w{true t yes y}.include?( self.downcase )
  end
end

class Fixnum
  def to_hex
    "%02x" % self
  end
end

class Control
  attr_accessor :id, :led, :group, :name, :ordinal, :switch
  
  def initialize( obj, group )
    @id = obj.id
    @name = obj.name
    @ordinal = obj.ordinal
    @switch = obj.switch
    @group = group
  end
  
  def ordinal_name
  end
end

class Fader < Control
  def self.midi_zero_byte
    0xe0
  end
  
  def self.mask_for_id( bytes )
    bytes[0] & 0b00001111
  end
end

class Button < Control
  def self.midi_zero_byte
    0x90
  end
  
  def self.mask_for_id( bytes )
    bytes[1]
  end
end

class Led < Control
end

class LedRing < Led
end

class Pot < Control
  def self.midi_zero_byte
    0xb0
  end
  
  def self.mask_for_id( bytes )
    bytes[1] & 0b00011111
  end

  def led=( rhs )
    @led = LedRing.new( rhs, group )
  end
end

class Group < Array
  attr_accessor :name, :controls
  
  def initialize( name )
    @name = name
  end
  
  def add_control( control )
    @controls ||= Array.new
    @controls << control
  end
end

class Strip < Group
  
  attr_accessor :ordinal
  def initialize( name, ordinal )
    super( name )
    @ordinal = ordinal
  end
  
  def name
    "#{@name}_#{@ordinal}"
  end
  
end

types = { 0xe0 => Fader, 0x90 => Button, 0xb0 => Pot }

# number of controls, name, switch, led, id
# anything that doesn't have the correct number
# of columns will be ignored
# actually, 'switch' means it generates data
# whereas 'led' means it receives data
$control_data = <<EOF
type,count,group,name,switch,led,id
# faders
fader,8,strip,gain,1,0,0x00
fader,1,master,gain,1,0,0x08

# pots
pot,8,strip,vpot,1,1,0x10
pot,1,,jog,1,0,0x3c
pot,1,,external,1,0,0x2e

# strip buttons
button,8,strip,recenable,1,1,0x0
button,8,strip,solo,1,1,0x08
button,8,strip,mute,1,1,0x10
button,8,strip,select,1,1,0x18
button,8,strip,vselect,1,0,0x20

# overlay buttons
button,1,assignment,io,1,1,0x28
button,1,assignment,sends,1,1,0x29
button,1,assignment,pan,1,1,0x2a
button,1,assignment,plugin,1,1,0x2b
button,1,assignment,eq,1,1,0x2c
button,1,assignment,dyn,1,1,0x2d
button,1,bank,left,1,0,0x2e
button,1,bank,right,1,0,0x2f
button,1,bank,channel_left,1,0,0x30
button,1,bank,channel_right,1,0,0x31
button,1,,flip,1,1,0x32
button,1,,edit,1,1,0x33

button,1,display,name/value,1,0,0x34
button,1,display,smpte/beats,1,0,0x35
button,1,,F1,1,0,0x36
button,1,,F2,1,0,0x37
button,1,,F3,1,0,0x38
button,1,,F4,1,0,0x39
button,1,,F5,1,0,0x3a
button,1,,F6,1,0,0x3b
button,1,,F7,1,0,0x3c
button,1,,F8,1,0,0x3d
button,1,,F9,1,0,0x3e
button,1,,F10,1,0,0x3f
button,1,,F11,1,0,0x40
button,1,,F12,1,0,0x41
button,1,,F13,1,0,0x42
button,1,,F14,1,0,0x43
button,1,,F15,1,0,0x44
button,1,,F16,1,0,0x45
button,1,modifiers,shift,1,0,0x46
button,1,modifiers,option,1,0,0x47
button,1,modifiers,control,1,0,0x48
button,1,modifiers,cmd/alt,1,0,0x49
button,1,automation,on,1,1,0x4a
button,1,automation,rec/ready,1,1,0x4b
button,1,functions,undo,1,1,0x4c
button,1,automation,snapshot,1,1,0x4d
button,1,automation,touch,1,1,0x4e
button,1,functions,redo,1,1,0x4f
button,1,functions,marker,1,1,0x50
button,1,functions,enter,1,1,0x51
button,1,functions,cancel,1,0,0x52
button,1,functions,mixer,1,0,0x53
button,1,transport,frm_left,1,1,0x54
button,1,transport,frm_right,1,1,0x55
button,1,transport,loop,1,1,0x56
button,1,transport,punch_in,1,1,0x57
button,1,transport,punch_out,1,1,0x58
button,1,transport,home,1,1,0x59
button,1,transport,end,1,1,0x5a

# transport buttons
button,1,transport,"rewind",1,1,0x5b
button,1,transport,"ffwd",1,1,0x5c
button,1,transport,"stop",1,1,0x5d
button,1,transport,"play",1,1,0x5e
button,1,transport,"record",1,1,0x5f
button,1,cursor,"cursor_up",1,0,0x60
button,1,cursor,"cursor_down",1,0,0x61
button,1,cursor,"cursor_left",1,0,0x62
button,1,cursor,"cursor_right",1,0,0x63
button,1,,"zoom",1,1,0x64
button,1,,"scrub",1,1,0x65
button,1,user,"user_a",1,0,0x66
button,1,user,"user_b",1,0,0x67

button,8,strip,"fader_touch",1,0,0x68
button,1,master,"fader_touch",1,0,0x70

button,1,,"smpte",0,1,0x71
button,1,,"beats",0,1,0x72
button,1,,"solo",0,1,0x73
button,1,,"relay_click",0,1,0x76
EOF

class Row
  attr_accessor :count, :name, :switch, :led, :start_id, :type, :group
  attr_accessor :id, :ordinal_name, :ordinal_group, :ordinal

  def initialize( hash )
    @count = hash['count'].to_i
    @name = hash['name']
    @switch = hash['switch'].to_b
    @led = hash['led'].to_b
    @start_id = hash['id'].hex
    @type = hash['type']
    @group = hash['group']
    
    @hash = hash
  end
  
  def each_ordinal( &block )
    for i in 0...count
      @ordinal = i + 1
      @ordinal_name = count > 1 ? "#{name}_#{ordinal}" : name
      @ordinal_group = count > 1 ? "#{group}_#{ordinal}" : group
      @id = start_id + i
      
      @hash['ordinal_name'] = @ordinal_name
      @hash['ordinal_group'] = @ordinal_group
      
      yield( self )
    end
    self
  end
  
  def to_hash
    @hash
  end
end

class Surface
  attr_reader :groups, :controls_by_id, :types, :midis, :controls
  
  def initialize
    @types = Hash.new
    @groups = Hash.new
    @controls = Array.new
    @controls_by_id = Hash.new
    @midis = Hash.new
  end
  
  def add_or_create_group( name, ordinal = nil )
    if name.nil?
      @groups['none'] = Group.new('none')
    else
      group = name =~ /strip/ ? Strip.new( name, ordinal ) : Group.new( name )
      @groups[group.name] ||= group
    end
  end

  def parse( control_data = $control_data )
    FasterCSV.parse( control_data, :headers => true ) do |csv_row|
      next if csv_row.entries.size < 5 || csv_row[0] =~ /^\s*#/ || csv_row['id'].nil?
      row = Row.new( csv_row )
      
      row.each_ordinal do |row|
        group = add_or_create_group( row.group, row.ordinal )
        if row.switch
          # for controls
          control = eval "#{row.type.capitalize}.new( row, group )"
          
          # for controls with leds
          control.led = Led.new( row, group ) if row.led
        else
          # for LED-only entries
          if row.led
            control = Led.new( row, group )
            control.led = control
          end
        end
        
        # add the new control to the various lookups
        @controls_by_id[row.id] = control
        @controls << control
        group << control
        
        # add incoming midi bytes
        if row.switch
          types[control.class.midi_zero_byte] = control.class
          midis[control.class.midi_zero_byte] ||= Hash.new
          midis[control.class.midi_zero_byte][row.id] = control
        end
      end
    end
    self
  end
end
